// --------------------------------------
// I2C_PICDHTtest
// DrG 2016
// --------------------------------------
//
// simply reads the humidity and temperature and displays repeatedly
// for all three DHT-22 devices connected to the PIC via  I2C
//
#include <Wire.h>
// globals
byte DHT[16];  // to hold/access the byte ARRAY on the PIC
// I2C address (set in the PIC code)
int address=0x19;
// RH and temperature storage
float TempCz1, TempFz1, RHz1, TempCz2, TempFz2, RHz2, TempCz3, TempFz3, RHz3;

void setup()
{
  Wire.begin();    // join bus as master
  Serial.begin(9600);
  Serial.println("PIC Triple DHT22 I2C Humidity and Temperature");
  resetPIC();      // probably optional but not a bad idea
  delay(3000);     // to let me open up the serial window
}  
 
void loop()
{
  get_DHT1();  
  get_DHT2();
  get_DHT3(); 
  Serial.println();
  delay(5000);    // every 5 seconds

}

void get_DHT1(){
  // DHT[0]=command byte - set to 2 by master to request a read of DHT #1
  //    and on return it will be 1 if success.
  // 5 bytes of DHT info are as follows:
  // DHT data
  // DHT[1] RH integral
  // DHT[2] RH decimal
  // DHT[3] Temp integral
  // DHT[4] Temp decimal
  // DHT[5} checksum is the sum of all four bytes AND 255
  Wire.beginTransmission(address);
  Wire.write(0); // buffer index pointer
  Wire.write(2); // command byte (2=read DHT22 #1)
  Wire.endTransmission();
  delay(50);     // wait for it to be done - 50 msec to start
  Wire.beginTransmission(address);
  Wire.write(0); // buffer index pointer
  Wire.endTransmission();
  Wire.requestFrom(address, 6);
  // dump the first 6 bytes into an array
  for (int count=0;count<6; count++)
    {
      DHT[count]=Wire.read();
    }
  Serial.println();  
  // check if the command byte indicates that all went well
  if (DHT[0]!=1){
    Serial.println("*** Read Error ***");
  // further error processing if you like
  } 
  // Humidity
  RHz1=DHT[1];
  RHz1*=256;
  RHz1+=DHT[2];
  RHz1/=10;
  // Temperature  
  TempCz1=DHT[3]&0x7f;
  TempCz1*=256;
  TempCz1+=DHT[4];
  TempCz1/=10;
  if(DHT[3]&0x80)
    {
    TempCz1*=-1;
    }
  TempFz1=TempCz1 * 9 / 5 +32; 
  
  // checksum
  if ((DHT[1]+DHT[2]+DHT[3]+DHT[4] & 255) != DHT[5])
   {
   Serial.print(" *** BAD CRC! ***  ");  
   } 
  else{
   Serial.print(" CRC is good  ");
   }
  Serial.print("ZONE 1 Temperature ");
  Serial.print(TempCz1);
  Serial.print(" (C)  ");
  Serial.print(TempFz1);
  Serial.print(" (F) ");
  Serial.print("  Humidity ");
  Serial.println(RHz1); 
}
void get_DHT2(){
  // PIC[0]=command byte - set to 3 by master to request a read of DHT #2
  //    and on return it will be 1 (success) or an error value
  // 5 bytes of DHT info are as follows:
  // DHT data
  // DHT[6] RH integral
  // DHT[7] RH decimal
  // DHT[8] Temp integral
  // DHT[9] Temp decimal
  // DHT[10] checksum is the sum of all four bytes AND 255
  Wire.beginTransmission(address);
  Wire.write(0); // buffer index pointer
  Wire.write(3); // command byte (3=read DHT22 #2)
  Wire.endTransmission();
  delay(50);     // wait for it to be done - 50 msec to start
  Wire.beginTransmission(address);
  Wire.write(0); // buffer index pointer
  Wire.endTransmission();
  Wire.requestFrom(address, 1);
  // update the status byte
  DHT[0]=Wire.read();
  //now get the 5 bytes of DHT data into an array
  Wire.beginTransmission(address);
  Wire.write(6); // buffer index pointer (point to data for DHT #2)
  Wire.endTransmission();
  Wire.requestFrom(address, 5);
  for (int count=6;count<11; count++)
    {
      DHT[count]=Wire.read();
    } 
  // check if the command byte indicates that all went well
  if (DHT[0]!=1){
    Serial.println("*** Read Error ***");
  // further error processing if you like
  }  
  // Humidity
  RHz2=DHT[6];
  RHz2*=256;
  RHz2+=DHT[7];
  RHz2/=10;
  // Temperature  
  TempCz2=DHT[8]&0x7f;
  TempCz2*=256;
  TempCz2+=DHT[9];
  TempCz2/=10;
  if(DHT[8]&0x80)
    {
    TempCz2*=-1;
    }
  TempFz2=TempCz2 * 9 / 5 +32;   
  // checksum
  if ((DHT[6]+DHT[7]+DHT[8]+DHT[9] & 255) != DHT[10])
   {
   Serial.print(" *** BAD CRC! ***  ");  
   }     
  else{
   Serial.print(" CRC is good  ");
   }
  Serial.print("ZONE 2 Temperature ");
  Serial.print(TempCz2);
  Serial.print(" (C)  ");
  Serial.print(TempFz2);
  Serial.print(" (F) ");
  Serial.print("  Humidity ");
  Serial.println(RHz2); 
}

void get_DHT3(){
  // PIC[0]=command byte - set to 4 by master to request a read of DHT #3
  //    and on return it will be 1 (success) or an error value
  // 5 bytes of DHT info are as follows:
  // DHT data
  // DHT[11] RH integral
  // DHT[12] RH decimal
  // DHT[13] Temp integral
  // DHT[14] Temp decimal
  // DHT[15] checksum is the sum of all four bytes AND 255
  Wire.beginTransmission(address);
  Wire.write(0); // buffer index pointer
  Wire.write(4); // command byte (4=read DHT22 #3)
  Wire.endTransmission();
  delay(50);     // wait for it to be done - 50 msec to start
  Wire.beginTransmission(address);
  Wire.write(0); // buffer index pointer
  Wire.endTransmission();
  Wire.requestFrom(address, 1);
  // update the status byte
  DHT[0]=Wire.read();
  //now get the 5 bytes of DHT data into an array
  Wire.beginTransmission(address);
  Wire.write(11); // buffer index pointer (point to data for DHT #2)
  Wire.endTransmission();
  Wire.requestFrom(address, 5);
  for (int count=11;count<16; count++)
    {
      DHT[count]=Wire.read();
    } 
  // check if the command byte indicates that all went well
  if (DHT[0]!=1){
    Serial.println("*** Read Error ***");
  // further error processing if you like
  }  
  // Humidity
  RHz3=DHT[11];
  RHz3*=256;
  RHz3+=DHT[12];
  RHz3/=10;
  // Temperature  
  TempCz3=DHT[13]&0x7f;
  TempCz3*=256;
  TempCz3+=DHT[14];
  TempCz3/=10;
  if(DHT[13]&0x80)
    {
    TempCz3*=-1;
    }
  TempFz3=TempCz3 * 9 / 5 +32; 
  
  // checksum
  if ((DHT[11]+DHT[12]+DHT[13]+DHT[14] & 255) != DHT[15])
   {
   Serial.print(" *** BAD CRC! ***  ");  
   }
  else{
   Serial.print(" CRC is good  ");
   }
  Serial.print("ZONE 3 Temperature ");
  Serial.print(TempCz3);
  Serial.print(" (C)  ");
  Serial.print(TempFz3);
  Serial.print(" (F) ");
  Serial.print("  Humidity ");
  Serial.println(RHz3); 
}

void resetPIC(){
    Wire.beginTransmission(address);
    Wire.write(0);    // buffer index pointer
    Wire.write(16);   // command byte (0x10= Reset Pic)
    Wire.endTransmission();
}  

